%%%%%%% Snowball Model used in Mills et al. (2011)
%%%%%%% Benjamin Mills. Dec 2012. benjaminmills@live.com
%%%%%%%
%%%%%%% Simplification of Minimal model to assess effects of transport limitation of
%%%%%%% weathering and snowball glaciations in the Neoproterozoic
%%%%%%% 
%%%%%%% Updated to include ocean crust carbonatization as a CO2 sink
%%%%%%% (see thesis chap. 6)
%%%%%%%
%%%%%%% Model frontend
%%%%%%%
%%%%%%% User panel line 165


%%%%%%% clear workspace
clear all

%%%%%%% set number of model runs to do, use q and w to make 2D parameter space.
%%%%%%% (number of runs =pxq)
qsteps=1;
wsteps=1;

%%%%%%% set up vectors and matrices to record solutions
stablephos=zeros(qsteps,1);
stableo=zeros(qsteps,1);
stablecarb=zeros(qsteps,1);
stablemeth=zeros(qsteps,1);
stabletemp=zeros(qsteps,wsteps);
e2=zeros(qsteps,1);
r2=zeros(qsteps,1);
mintemp=zeros(qsteps,wsteps);
oxymatch=zeros(qsteps,wsteps);
timetostable=zeros(qsteps,wsteps);

%%%%%%% set up globals
global oxidwlimit
global trigtemp
global escapetemp
global albedo_base
global albedo_ice
global icelimit
global snowb
global glaciate
global silcap
global carbcap
global dmocb
global rhobase
global rhoup
global dbase
global dup
global ebase
global eup
global telltime
global alphaOCC
global k1
global k2
global k3
global k4
global k5
global k7
global k8
global k9
global k10
global a
global b
global d
global rp
global P0
global O0
global A0
global G0
global C0
global Q0
global M0
global Sfixed
global notesteady
global steadytime
global kcarbw
global silcapflour
global carbcapflour
global flourmax
global silcapREAL
global carbcapREAL
global flourbase
global steadytemptolerance   
global whenstart
global whenend
global OCCLIMIT
global kocc


%%%%%%% model parameters

k1=9*10^12;  %%% COPSE mocb + locb
k2=1.5*10^10;
k3=6*10^9;
k4=0.86;
k9=1.25*10^12;
k10=6.65*10^12;
kcarbw=1.335*10^13;

%%%%%%% ocean crust carbonatization rate at present day
kocc = 1.65*10^12;
% kocc=0; %run without consideration of OCC (as in Mills et al. 2011)

%%%%%%% calculate the following constants for steady state:
k7=k1-k9;
k5=k7+k9+k10-k1 - kocc; %%% amended for OCC by adding last number
k8=k1/250  +k2 +k3;

% R_PDB
rp=0.0112372;
dmocb=-30;

%reservoir present day sizes
P0=3.1*10^15;
O0=3.7*10^19;
A0=3.193*10^18;
G0=1.25*10^21;
C0=5*10^21;
Q0=1.3*10^20;
M0=10^14;


%%%%%%% define functions used in plots

%%%%%%% degassing rate
degass=inline('(x-w).*(1./(1+exp(-2*(3*10^-7).*(z-3.75*10^9))))+w   +y.*0');  % f(rhobase,rhoup,0,t) 

%%%%%%% weathering enhancement
rho=inline('(x-w).*(1./(1+exp(-2*(6*10^-8).*(z-3.75*10^9))))+w   +  (w-x).*(1./(1+exp(-2*(6*10^-8).*(z-3.92*10^9))))      +y.*0');  % f(rhobase,rhoup,0,t) 

%%%%%%% atfrac(A), valid for albedo={0.3,0.6} only
atfrac=inline(' 0.8.*( x./(x  +  (    1  +   5.*( ( b-0.3 )./ 0.35 )    ).*10^20  ) )  ');

%%%%%%% temp(Sfixed,albedo,CO2atm) (LBL)
Temp=inline('        ((   (a.*(1-b))./(4.*(1-(0.773./2)).*5.67*10^-8)  ).^0.25)          + 0.1815.*(        0.2507.*((log10(x)).^4)  +  3.9216.*((log10(x)).^3) + 23.8113.*((log10(x)).^2) + 83.4113.*((log10(x))) + 131.6138        )  ');

%%%%%%% ocean anoxic fraction (truncated)
anox=inline('(0.*(1 - 0.5.*(sign((1-((0.86*(x./(3.7*10^19)))./(y./(3.1*10^15))))) + 1)) + (1-((0.86.*(x./(3.7*10^19)))./(y./(3.1*10^15)))).*0.5.*(sign((1-((0.86*(x./(3.7*10^19)))./(y./(3.1*10^15)))))+1 ))');

%%%%%%% C to P burial ratio
CPsea=inline('250');  

%%%%%%% CALCULATE COMPOSITION USING C12 VALUE (comp=d13C value) comp(C12,totalC)
comp=inline('((((y-x)./(x))./0.0112372)-1).*1000');
%%%%%%% reverse
amountc12=inline('(x/(1+  y*( (z/1000) +1 )  ) )');  %% (x/(1+  y*( (z/1000) +1 )  ) )


%%%%%%% dmccb no fractionaton
dmccbnofrac=inline('x.*z.*0 +1');

%%%%%%% variable dmccb
dmccb=inline(  '  x.*( (9483./z) -23.89) +15.1 -4232./z  ');     % f(atfrac,tempK)    

    

%%%%%%% loop for multiple runs
for w=1:wsteps

for q=1:qsteps


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% USER PANEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    
    
%%%%%%% fix solar constant   
Sfixed = 1298 ;

%%%%%%% basic dependencies
a=0; %oxidative weathering dependency on O2 concentration
b=2; %marine organic carbon burial dependency on new production
d=0; %organic carbon degassing dependency on O2 concentration

%%%%%%% rho (biological weathering enhancement factor)
rhobase=1;
rhoup=4;

%%%%%%% degassing rate
dbase=1;
dup=1;

%%%%%%% nutrient weathering efficiency
ebase=1;
eup=1;

%%%%%%% glacial proterties
trigtemp=283;
escapetemp=263;
albedo_base=0.3;
albedo_ice=0.6;
icelimit=2; %% silicate and carbonate weathering reduction (in orders of magnitude) when in snowball
oxidwlimit=2; %% reduce oxidative weathering also?


%%%%%%% weathering rate maximum when rock flour is present (le hir 2009)
silcapflour = 10;
carbcapflour = 10;

%%%%%%% define maximum amount of carbon removal due to rock flour weathering
flourbase = 10^14 ;
flourmax = 10^18;


%%%%%%% cap silicate weathering rate?
silcap=1.4 ;

%%%%%%% variable cap for steady state fig
% silcap= 10^( (q-1)/66  )  ;
% silcap2(q,w)=silcap ;

%%%%%%% maximum carbonate weathering assumed to be same value
carbcap = silcap;


%%%%%%% alpha parameter for OCC
alphaOCC = 0.23;

%%%%%%% limit OCC RATE?
OCCLIMIT = 2;

%%%%%%% model timeframe in years (present day = 4.5e9)
whenstart = 3.7*10^9; 
whenend = 4*10^9;

%%%%%%% display results graphs % 1 = normal, 2 = run down from start
display = 1; 

%%%%%%% display timestep as model runs?
telltime = 1; 

%%%%%%% note when it reaches stability (for rundowns)
notesteady = 2; 
% 0 = note the steady state time, 2 = do not note. (notesteady = 1 forbidden).

%%%%%%% steady temperature tolerance (K)
steadytemptolerance  = 0.01 ; 



%%%%%%% reservoir starting sizes
qstart=0;
bstart=0;
gstart=G0;
cstart=C0;
flourstart=flourbase;

%steady state at 800Ma
pstart=2.720957621444806e+15;
ostart=0.05*3.7*10^19;   %% 1% atm
astart=3.899990316164868e+18;

%%%%%%% for starting with ~0.3 atm CO2.
% astart=1.687*10^20;

%%%%%%% set maxstep
options = odeset('maxstep',100000) ;




%%%%%%% rune the system
[T,Y] = ode15s(@snowball_equations,[whenstart whenend],[pstart ostart astart flourstart gstart cstart 0 amountc12(astart,rp,0) 0 amountc12(gstart,rp,-30) amountc12(cstart,rp,0) amountc12(0,rp,-30) 0 0 albedo_base 0 0 amountc12(astart,rp,0) amountc12(gstart,rp,-30) amountc12(cstart,rp,0)  0  0  288 280/1000000 0],options);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF USER PANEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    









%%%%%%%%%%%%  record steady state details

A=size(Y);
lastval=A(1);

stabletemp(q,w)=Temp(Sfixed, Y(lastval,15), (Y(lastval,3)./A0).*(atfrac( Y(lastval,15) , Y(lastval,3))./atfrac( 0.3 , A0)).*(280/1000000) );
stablecarb(q,w)=Y(lastval,3);
stableo(q,w)=Y(lastval,2);
stablephos(q,w)=Y(lastval,1);

if notesteady == 0
timetostable(q,w) = 0 ;
end

if notesteady == 1
timetostable(q,w) = steadytime - (whenstart) ;
end

if notesteady == 2
timetostable(q,w) = 0 ;
end


end

end








%%%%%%% plotting scripts to display single run


A=size(Y);
w=A(1);
 
% % O2(ATM)
for u=1:w
    o2atm(u,1)=Y(u,2)*((0.21)/(3.7*10^19));
end

% % CO2 (ATM)
for u=1:w
    co22(u,1)=(Y(u,3)./A0).*(atfrac( Y(u,15) , Y(u,3))./atfrac( 0.3 , A0)).*(280/1000000);
end




%%%%%%% display 1: x axis unchanged.

if display==1 
   

subplot(5,1,1);
plot(T,rho(rhobase,rhoup,0,T),'displayname','RHO')
hold on
ylabel('Rho forcing (relative)')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 
title('Key parameters')

subplot(5,1,2);
plot(T,Y(:,24),'displayname','CO2 (atm)')
ylabel('CO2 (atm)')
hold on
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

subplot(5,1,3);
plot(T,Y(:,21),'r','displayname','fCO2') 
hold on
plot(T,Y(:,22),'c','displayname','gCO2') 
ylabel('relative weathering fluxes')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 


subplot(5,1,4);
plot (T, Y(:,23),'displayname','temp')
hold on
ylabel('Temp(K)')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % %


subplot(5,1,5);
plot(T,comp(Y(:,8),Y(:,3))+dmccb(atfrac( Y(:,15) , Y(:,3)),Temp(Sfixed, Y(:,15), (Y(:,3)/A0).*(atfrac( Y(:,15) , Y(:,3))./atfrac( 0.3 , A0)).*(280/1000000) )),'k','displayname','d13C (new carbonate) temp dep ON')
hold on
plot(T,comp(Y(:,18),Y(:,3))+dmccbnofrac(atfrac( Y(:,15) , Y(:,3)),Temp(Sfixed, Y(:,15), (Y(:,3)/A0).*(atfrac( Y(:,15) , Y(:,3))./atfrac( 0.3, A0)).*(280/1000000) )),'k--','displayname','d13C (new carbonate) temp dep OFF')
ylabel('Fractionation (permille)')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % %



%%%%%%% isotope system with variable mccb fractionation
figure(2)
plot(T,comp(Y(:,8),Y(:,3))+dmccb(atfrac( Y(:,15) , Y(:,3)),Temp(Sfixed, Y(:,15), (Y(:,3)/A0).*(atfrac( Y(:,15) , Y(:,3))./atfrac( 0.3 , A0)).*(280/1000000) )),'k.','displayname','d13C (new carbonate)')
hold on
plot(T,comp(Y(:,8),Y(:,3)),'r','displayname','d13C (atmosphere/ocean CO2)')
plot(T,comp(Y(:,10),Y(:,5)),'g:','displayname','d13C (total organic)')
plot(T,comp(Y(:,11),Y(:,6)),'k:','displayname','d13C (total carbonate)')
plot(T,comp(Y(:,8),Y(:,3)) +dmocb,'g.','displayname','d13C (new organic)')
title('variable mccb fractionation')
ylabel('fractionation (permille)')


figure(3)
plot(T,       ( (k1.*((Y(:,1)./P0).^b))./CPsea(anox(Y(:,2),Y(:,1))) ),'b','displayname','MOPB') ;
hold on
plot(T,       k2.*(   (k1.*((Y(:,1)/P0).^b))./(k1) )    ,'c','displayname','CaPB') ;
plot(T,       (k3./k4).*(1-anox(Y(:,2),Y(:,1))).*(  ( (k1.*((Y(:,1)./P0).^b))  )./(k1) )  ,'r','displayname','FePB') ;
plot(T, ( (k1.*((Y(:,1)./P0).^b))./CPsea(anox(Y(:,2),Y(:,1))) ) + k2.*(   (k1.*((Y(:,1)/P0).^b))./(k1) )  +  (k3./k4).*(1-anox(Y(:,2),Y(:,1))).*(  ( (k1.*((Y(:,1)./P0).^b))  )./(k1) ) ,'k','displayname','total') ;
title('P burial')
ylabel('mol/yr')

% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % plot A

figure(4)
semilogy(T,Y(:,3),'displayname','Ocean/atmosphere carbon (mol)')
hold on
title('Hydrospheric carbon')
ylabel('Carbon (mol)')

%%%%%%% isotope system with fixed mccb fractionation
figure(5)
plot(T,comp(Y(:,18),Y(:,3))+dmccbnofrac(atfrac( Y(:,15) , Y(:,3)),Temp(Sfixed, Y(:,15), (Y(:,3)/A0).*(atfrac( Y(:,15) , Y(:,3))./atfrac( 0.3 , A0)).*(280/1000000) )),'k.','displayname','d13C (new carbonate)')
hold on
plot(T,comp(Y(:,18),Y(:,3)),'r','displayname','d13C (atmosphere/ocean CO2)')
plot(T,comp(Y(:,19),Y(:,5)),'g:','displayname','d13C (total organic)')
plot(T,comp(Y(:,20),Y(:,6)),'k:','displayname','d13C (total carbonate)')
plot(T,comp(Y(:,18),Y(:,3)) +dmocb,'g.','displayname','d13C (new organic)')
title('fixed mccb fractionation')
ylabel('Fractionation (permille)')


figure(6)
plot(T,Y(:,4),'displayname','Flour (mol C equiv.)')
title('Rock flour')
ylabel('mol C equiv.')

end




%%%%% display 2: t starts from 0 at initiation time.


if display==2 
 


subplot(5,1,1);
semilogy(T-(3.7*10^9),Y(:,3))
hold on
ylabel('A (mol)')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % 

subplot(5,1,2);
plot(T-(3.7*10^9),Y(:,24),'displayname','CO2 (atm)')
hold on
ylabel('pCO2 (atm)')
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 


subplot(5,1,3);
plot(T-(3.7*10^9),Y(:,21),'r') 
hold on
plot(T-(3.7*10^9),Y(:,22),'c') 
plot(T-(3.7*10^9),Y(:,25)./(1.65*10^12),'k') 
ylabel({'relative weathering fluxes';'silw (red), carbw (cyan), OCC (black)'})
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % 


subplot(5,1,4);
plot (T-(3.7*10^9), Y(:,23),'displayname','temp')
hold on
ylabel('Temp (K)')
% % % % % % % % % % % % % % % % % % % % % % % %


subplot(5,1,5);
plot(T-(3.7*10^9),Y(:,4) - flourbase)
ylabel('Flour (mol C equiv.)')
% % % % % % % % % % % % % % % % % % % % % % % %


end



% % % % % % CARBON CONSERVATION

totalcarbon=(Y(A(1),3)+Y(A(1),5)+Y(A(1),6)-(Y(1,3)+Y(1,5)+Y(1,6)))

totalc12_ghost1=(Y(A(1),8)+Y(A(1),10)+Y(A(1),11)-(Y(1,8)+Y(1,10)+Y(1,11)))
totalc12_ghost2=(Y(A(1),18)+Y(A(1),19)+Y(A(1),20)-(Y(1,18)+Y(1,19)+Y(1,20)))

